unit JimBeerForJim;

interface

uses
  Classes;

type
  // Declaration of type to use as event handler. Note use of "as object"
  TJimOnGetValue = procedure (Sender : TObject;var Value : string) of object;

  TJimBeerForJim = class(TComponent)
  private
    FBeerName       : string;
    FCaseCount      : Integer;
    FCardNumber     : string;
    FCardExpiryDate : string;
    FOnGetEmailBody : TJimOnGetValue;
    FEmailTitle     : string;

    function  GetEmailBody : string;
    procedure SetCaseCount(const Value : Integer);
  protected
    function GetEmailAddress : string; virtual;

    // To change this read-only value, override the constructor (or the
    // Loaded procedure)
    property EmailTitle : string read FEmailTitle;
  public
    constructor Create(AOwner: TComponent); override;

    procedure Execute;

    // Properties do not have to be published, and can be read-only
    property EmailAddress : string read GetEmailAddress;
  published
    property BeerName       : string read FBeerName write FBeerName;
    // Use a setter method to enforce contraints on the case count
    property CaseCount      : Integer read FCaseCount write SetCaseCount default 1;
    property CardNumber     : string read FCardNumber write FCardNumber;
    property CardExpiryDate : string read FCardExpiryDate
                                     write FCardExpiryDate;

    // Event handler - note naming convention
    // Can have public events, but the event handler assignments will not
    // get saved to the .dfm file
    property OnGetEmailBody : TJimOnGetValue read FOnGetEmailBody
                                             write FOnGetEmailBody;
  end;


implementation

uses
  ShellApi, SysUtils, Windows;

{ TJimBeerForJim }

constructor TJimBeerForJim.Create(AOwner : TComponent);
begin
  inherited;
  // Can use fields to set default values as they belong to this component
  // cf setting property values in TJImBeerLabel constructor
  FBeerName   := 'Hoegaarden';
  // Note that we must set the default value here, as the default directive on
  // the property declaration only determines whether a value is saved in the
  // form file or not.
  FCaseCount  := 1;
  FEmailTitle := 'Beer for Jim';
end;


procedure TJimBeerForJim.SetCaseCount(const Value : Integer);
begin
  // Just in case there are any skinflints, enforce the case count to be
  // at least one
  if Value > 0 then begin
    FCaseCount := Value;
  end;
end;


function TJimBeerForJim.GetEmailBody : string;
begin
  Result := Format('Send %d cases of %s to Jim Cooper, ' +
                   'Charge to card number %s, expiry %s',
                   [CaseCount,BeerName,CardNumber,CardExpiryDate]);

  // Use event handler to change value if needed
  if Assigned(FOnGetEmailBody) then begin
    FOnGetEmailBody(Self,Result);
  end;
end;


function TJimBeerForJim.GetEmailAddress : string;
begin
  // Can override this function in subclasses
  Result := 'sales@eurobeer.co.uk';
end;


procedure TJimBeerForJim.Execute;
  var
    MailTo : string;
begin
  // The EmailXXX properties demonstrate the different ways we can use to get
  // values in components and how to change them in descendants
  MailTo := Format('mailto:%s?subject=%s&body=%s',[EmailAddress,
                                                   EmailTitle,
                                                   GetEmailBody]);
  ShellExecute(0,'open',PChar(MailTo),'','',SW_MAXIMIZE);
end;


end.
